// filename *************************heap_test.c ************************
// Test for memory heap for dynamic memory allocation.

// Jacob Egner 2008-07-31
// modified 8/31/08 Jonathan Valvano for style
// modified 12/16/11 Jonathan Valvano for 32-bit machine

/* This example accompanies the book
   Embedded Systems: Real-Time Operating Systems for the Arm Cortex-M3, Volume 3,  
   ISBN: 978-1466468863, Jonathan Valvano, copyright (c) 2012

   Program xx.x, section xx.x

 Copyright 2012 by Jonathan W. Valvano, valvano@mail.utexas.edu
    You may use, edit, run or distribute this file
    as long as the above copyright notice remains

 THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 For more information about my classes, my research, and my books, see
 http://users.ece.utexas.edu/~valvano/
 */


#include "heap.h"

// global so easier to see with the debugger
// Proper style would be to make these variables local to main.
long status; 
long* ptr;
long* p1;
long* p2;
long* p3;
unsigned char* q1;
unsigned char* q2;
unsigned char* q3;
unsigned char* q4;
unsigned char* q5;
unsigned char* q6;
long maxBlockSize;
unsigned char* bigBlock;
heap_stats_t stats;

int main(){
  
  long i;
  

  status = Heap_Init();

  ptr = Heap_Malloc(sizeof(long));
  *ptr = 0x1111;
  status = Heap_Test();

  status = Heap_Free(ptr);
  status = Heap_Test();

  ptr = Heap_Malloc(1);
  status = Heap_Test();

  status = Heap_Free(ptr);
  status = Heap_Test();

  p1 = (long*) Heap_Malloc(1 * sizeof(long));
  p2 = (long*) Heap_Malloc(2 * sizeof(long));
  p3 = (long*) Heap_Malloc(3 * sizeof(long));
  p1[0] = 0xAAAA;
  p2[0] = 0xBBBB;
  p2[1] = 0xBBBB;
  p3[0] = 0xCCCC;
  p3[1] = 0xCCCC;
  p3[2] = 0xCCCC;
  status = Heap_Test();
  stats = Heap_Stats();

  status = Heap_Free(p1);
  status = Heap_Free(p3);
  status = Heap_Test();

  status = Heap_Free(p2);
  status = Heap_Test();
  stats = Heap_Stats();

  for(i = 0; i <= HEAP_SIZE_WORDS; i++){
    ptr = Heap_Malloc(sizeof(long));
  }
  status = Heap_Test();
  stats = Heap_Stats();

  status = Heap_Init();
  q1 = Heap_Malloc(1);
  q2 = Heap_Malloc(2);
  q3 = Heap_Malloc(3);
  q4 = Heap_Malloc(4);
  q5 = Heap_Malloc(5);
  status = Heap_Test();

  *q1 = 0xDD;
  q6 = Heap_Realloc(q1, 6);
  status = Heap_Test();
  stats = Heap_Stats();

  //q1 should point to freed space
  status = Heap_Free(q1);
  status = Heap_Test();

  for(i = 0; i < 6; i++){
    q6[i] = 0xEE;
  }
  q1 = Heap_Realloc(q6, 2);
  status = Heap_Test();

  status = Heap_Init();
  maxBlockSize = HEAP_SIZE_BYTES - 2 * sizeof(long);
  bigBlock = Heap_Malloc(maxBlockSize);
  for(i = 0; i < maxBlockSize; i++){
    bigBlock[i] = 0xFF;
  }
  status = Heap_Test();
  stats = Heap_Stats();
  status = Heap_Free(bigBlock);
  bigBlock = Heap_Calloc(maxBlockSize);
  status = Heap_Test();
  stats = Heap_Stats();
  for(;;){
  } 
}

